#!/usr/bin/env bash
# @Project      : AI @by PyCharm
# @Time         : 2023/3/23 14:40
# @Author       : betterme
# @Email        : 313303303@qq.com
# @Software     : PyCharm
# @Description  :

#streamlit run chatmind.py
#streamlit run chatfile_nesc_v1.py
#streamlit run chatbook.py

streamlit run  chatfile_nesc_last.py


